local TEXTURE = Resources.load_texture("hypnobeam.png")
local sfx = Resources.load_audio("pulse.ogg")

function card_init(player, props)
	-- Create an action with an animation state
	local action = Action.new(player, "CHARACTER_SWING")

	action.on_execute_func = function(self, user)
		-- Obtain the direction the user is facing
		local direction = user:facing()

		-- Second frame creates visual
		self:add_anim_action(2, function()
			-- Create the hilt attachment
			local hilt = self:create_attachment("HILT")
			local hilt_sprite = hilt:sprite()
			hilt_sprite:set_texture(user:texture())
			hilt_sprite:set_layer(-2)
			hilt_sprite:use_root_shader(true)

			-- Animate the hilt with the hand state.
			local hilt_anim = hilt:animation()
			hilt_anim:copy_from(user:animation())
			hilt_anim:set_state("HAND")
			hilt_anim:apply(hilt_sprite)
		end)

		-- Third frame begins the attack
		self:add_anim_action(3, function()
			local spell = Spell.new(user:team())
			spell:set_tile_highlight(Highlight.Solid)
			spell:set_facing(direction)
			spell:set_hit_props(
				HitProps.new(
					props.damage,
					props.hit_flags,
					props.element,
					user:context(),
					Drag.None
				)
			)
			spell.slide_started = false
			spell.on_update_func = function(self)
				local tile = self:current_tile()
				if not tile or tile:is_edge() then self:delete() end
				tile:attack_entities(self)
				local dest = self:get_tile(direction, 1)
				local ref = self
				self:slide(dest, 24, function() ref.slide_started = true end)
			end
			-- Make the spell show up
			spell:set_texture(TEXTURE, true)

			-- Animate it
			local fx_anim = spell:animation()
			fx_anim:load("hypnobeam.animation")

			-- The animation state we'll be using.
			fx_anim:set_state("DEFAULT")

			-- Makes the animation loop
			fx_anim:set_playback(Playback.Loop)

			-- Offset from the ground. Not sure why I did it like this.
			local offset = spell:sprite():offset()
			spell:set_offset(offset.x * 0.5, offset.y - 40.0 * 0.5)

			spell.on_collision_func = function(self, other)
				-- Delete spell if it collides with anything so that it dissipates on barriers.
				self:delete()
			end

			spell.on_attack_func = function(self, other) --[[no special ability on dealing damage--]] end

			-- It can move anywhere. Broken tiles and so on are not an issue. Just delete if it collides with a hurtbox.
			spell.can_move_to_func = function(tile)
				return true
			end

			-- Play the sound when spawned
			spell.on_spawn_func = function(self)
				Resources.play_audio(sfx)
			end

			-- Obtain a tile to spawn the attack on
			local tile = user:get_tile(direction, 1)

			-- Spawn it
			player:field():spawn(spell, tile)
		end)
	end
	return action
end
